@@ -10,15 +10,20 @@ import sqlite3
 
 import pandas as pd
 
-from predectorutils.indexedresults import ResultsTable, TargetRow, ResultRow
+from predectorutils.database import (
+    load_db,
+    ResultsTable,
+    ResultRow,
+    TargetRow
+)
 
 
 def cli(parser: argparse.ArgumentParser) -> None:
 
     parser.add_argument(
-        "infile",
-        type=argparse.FileType('r'),
-        help="The ldjson file to parse as input. Use '-' for stdin."
+        "db",
+        type=str,
+        help="Where to store the database"
     )
 
     parser.add_argument(
@@ -32,10 +37,13 @@ def cli(parser: argparse.ArgumentParser) -> None:
     )
 
     parser.add_argument(
-        "--db",
-        type=str,
-        default=":memory:",
-        help="Where to store the database"
+        "--mem",
+        type=float,
+        default=1.0,
+        help=(
+            "The amount of RAM in gibibytes to let "
+            "SQLite use for cache."
+        )
     )
 
     return
@@ -52,22 +60,21 @@ def fetch_targets(
         f"FROM {table}"
     ))
     for r in result:
-        yield TargetRow(*r)
+        yield TargetRow.from_rowfactory(r)
     return
 
 
-def inner(con: sqlite3.Connection, args: argparse.Namespace) -> None:
-    # This thing just keeps the contents on disk.
-    # Helps save memory.
-    cur = con.cursor()
+def inner(
+    con: sqlite3.Connection,
+    cur: sqlite3.Cursor,
+    args: argparse.Namespace
+) -> None:
+    from ..analyses import Analyses
 
     tab = ResultsTable(con, cur)
-    tab.create_tables()
-    tab.insert_results(ResultRow.from_file(args.infile))
-
     targets = list(fetch_targets(tab, "results"))
 
-    seen: Set[str] = set()
+    seen: Set[Analyses] = set()
     for target in targets:
         if target.analysis in seen:
             raise ValueError(
@@ -76,10 +83,10 @@ def inner(con: sqlite3.Connection, args: argparse.Namespace) -> None:
         else:
             seen.add(target.analysis)
 
-        records = tab.select_target(target, "results")
+        records = tab.select_target(target, checksums=False)
         df = pd.DataFrame(map(lambda x: x.as_analysis().as_series(), records))
 
-        fname = args.template.format(analysis=target.analysis)
+        fname = args.template.format(analysis=str(target.analysis))
         dname = os.path.dirname(fname)
         if dname != '':
             os.makedirs(dname, exist_ok=True)
@@ -88,9 +95,9 @@ def inner(con: sqlite3.Connection, args: argparse.Namespace) -> None:
 
 
 def runner(args: argparse.Namespace) -> None:
-    con = sqlite3.connect(args.db)
     try:
-        inner(con, args)
+        con, cur = load_db(args.db, args.mem)
+        inner(con, cur, args)
     except Exception as e:
         raise e
     finally:
