@@ -8,14 +8,13 @@ from typing import Iterator
 from typing import Tuple
 from typing import TextIO
 from typing import List, Set, Dict
-from typing import Optional
 
 from Bio import SeqIO
 from Bio.SeqRecord import SeqRecord
 from Bio.SeqUtils.CheckSum import seguid
 
 from predectorutils.analyses import Analyses
-from predectorutils.indexedresults import TargetRow, ResultsTable, ResultRow
+from predectorutils.database import load_db, TargetRow, ResultsTable, ResultRow
 
 
 def cli(parser: argparse.ArgumentParser) -> None:
@@ -24,21 +23,7 @@ def cli(parser: argparse.ArgumentParser) -> None:
         "-o", "--outfile",
         type=argparse.FileType('w'),
         help="Where to write the precomputed ldjson results to.",
-        default="-",
-    )
-
-    parser.add_argument(
-        "-p", "--precomputed",
-        type=argparse.FileType('r'),
         default=None,
-        help="The ldjson to parse as precomputed input."
-    )
-
-    parser.add_argument(
-        "--db",
-        type=str,
-        default=":memory:",
-        help="Where to store the sqlite database"
     )
 
     parser.add_argument(
@@ -51,6 +36,22 @@ def cli(parser: argparse.ArgumentParser) -> None:
         )
     )
 
+    parser.add_argument(
+        "--mem",
+        type=float,
+        default=1.0,
+        help=(
+            "The amount of RAM in gibibytes to let "
+            "SQLite use for cache."
+        )
+    )
+
+    parser.add_argument(
+        "db",
+        type=str,
+        help="Where the sqlite database is"
+    )
+
     parser.add_argument(
         "analyses",
         type=argparse.FileType('r'),
@@ -88,30 +89,45 @@ def get_checksum_to_ids(seqs: Dict[str, SeqRecord]) -> Dict[str, Set[str]]:
     return d
 
 
-def filter_seqs_by_done(
-    seqs: Iterator[SeqRecord],
-    an: Tuple[str, str, Optional[str]],
-    checksums: Dict[str, str],
-    done: Dict[Tuple[str, str, Optional[str]], Set[str]],
-) -> Iterator[SeqRecord]:
-    if an not in done:
-        return seqs
+def write_remaining_seqs(
+    remaining: List[str],
+    seqs: Dict[str, SeqRecord],
+    target: TargetRow,
+    checksum_to_ids: Dict[str, Set[str]],
+    template: str
+) -> None:
+    if len(remaining) == 0:
+        return
 
-    for seq in seqs:
-        chk = checksums[seq.id]
-        if chk not in done[an]:
-            yield seq
+    fname = template.format(**target.as_dict())
+
+    dname = os.path.dirname(fname)
+    if dname != '':
+        os.makedirs(dname, exist_ok=True)
+
+    buf: List[SeqRecord] = []
+
+    with open(fname, "w") as handle:
+        for checksum in remaining:
+            for id_ in checksum_to_ids[checksum]:
+                buf.append(seqs[id_])
+
+            if len(buf) > 10000:
+                SeqIO.write(buf, handle, "fasta")
+                buf = []
+
+        if len(buf) > 0:
+            SeqIO.write(buf, handle, "fasta")
     return
 
 
-def fetch_local_precomputed(
-    tab: ResultsTable,
-    checksums: Set[str],
+def write_results(
+    results: Iterator[ResultRow],
     outfile: TextIO
 ) -> None:
     buf: List[str] = []
-    for precomp in tab.select_checksums(checksums, "subset"):
-        buf.append(precomp.as_str())
+    for result in results:
+        buf.append(result.as_str())
 
         if len(buf) > 10000:
             print("\n".join(buf), file=outfile)
@@ -122,57 +138,18 @@ def fetch_local_precomputed(
     return
 
 
-def write_remaining_seqs(
-    seqs: Dict[str, SeqRecord],
-    tab: ResultsTable,
-    checksums: Set[str],
-    checksum_to_ids: Dict[str, Set[str]],
-    template: str
+def inner(
+    con: sqlite3.Connection,
+    cur: sqlite3.Cursor,
+    args: argparse.Namespace
 ) -> None:
-    for an, chks in tab.find_remaining(checksums, "subset"):
-        analysis, sversion, dversion = an
-
-        fname = template.format(analysis=analysis)
-        dname = os.path.dirname(fname)
-        if dname != '':
-            os.makedirs(dname, exist_ok=True)
-
-        these = []
-        for chk in chks:
-            for id_ in checksum_to_ids[chk]:
-                these.append(seqs[id_])
-
-        if len(these) > 0:
-            SeqIO.write(these, fname, "fasta")
-    return
-
-
-def filter_results_by_database_version(
-    results: Iterator[ResultRow],
-    requires_database: Set[str],
-) -> Iterator[ResultRow]:
-    for result in results:
-        if (
-            (result.analysis in requires_database) and
-            (result.database_version is None)
-        ):
-            continue
-        else:
-            yield result
-    return
-
-
-def inner(con: sqlite3.Connection, args: argparse.Namespace) -> None:
     seqs: Dict[str, SeqRecord] = SeqIO.to_dict(
         SeqIO.parse(args.infasta, "fasta")
     )
     checksum_to_ids = get_checksum_to_ids(seqs)
     checksums = set(checksum_to_ids.keys())
 
-    cur = con.cursor()
-
     tab = ResultsTable(con, cur)
-    tab.create_tables()
 
     requires_database = {
         str(a)
@@ -181,31 +158,35 @@ def inner(con: sqlite3.Connection, args: argparse.Namespace) -> None:
         if (a.get_analysis().database is not None)
     }
 
-    if args.precomputed is not None:
-        results = filter_results_by_database_version(
-            ResultRow.from_file(
-                args.precomputed,
-                replace_name=True
-            ),
-            requires_database
+    tab.insert_checksums(checksums)
+
+    for target in TargetRow.from_file(args.analyses):
+        if (
+            (target.analysis in requires_database) and
+            (target.database_version is None)
+        ):
+            continue
+
+        if args.outfile is not None:
+            local_results = tab.select_target(target, checksums=True)
+            write_results(local_results, args.outfile)
+
+        remaining_checksums = list(tab.find_remaining(target))
+
+        write_remaining_seqs(
+            remaining_checksums,
+            seqs,
+            target,
+            checksum_to_ids,
+            args.template
         )
-        tab.insert_results(results)
-
-    tab.deduplicate_table()
-
-    tab.insert_targets(TargetRow.from_file(args.analyses))
-
-    fetch_local_precomputed(tab, checksums, args.outfile)
-
-    # Write out remaining tasks to be done
-    write_remaining_seqs(seqs, tab, checksums, checksum_to_ids, args.template)
     return
 
 
 def runner(args: argparse.Namespace) -> None:
-    con = sqlite3.connect(args.db)
     try:
-        inner(con, args)
+        con, cur = load_db(args.db, args.mem)
+        inner(con, cur, args)
     except Exception as e:
         raise e
     finally:
